SUBROUTINE PressureHorizAndStringDistribution
    
  
    !!      Record of revisions
    !!          Date            Programmer          Discription of change
    !!         ------          ------------        -----------------------
    !!        1396/07/30        Sheikh              Original code
    !! 
    
    USE FricPressDropVars
    USE PressureDisplayVARIABLES
    USE MudSystemVARIABLES
    USE GeoElements_FluidModule
    USE Fluid_Flow_Startup_Vars
    USE KickVariables
    USE CMudPropertiesVariables
    USE CDataDisplayConsoleVariables , StandPipePressureDataDisplay=> StandPipePressure
    USE CDataDisplayConsoleVariables , CasingPressureDataDisplay=> CasingPressure
    USE CDrillWatchVariables
    USE CShoeVariables
    USE CDownHoleVariables , CasingPressureDownhole => CasingPressure
    USE TD_WellGeometry
    USE CManifolds
    USE VARIABLES
    USE CError
    USE UTUBEVARS
    USE CKellyConnectionEnumVariables
    USE Pump_VARIABLES
    USE , INTRINSIC :: IEEE_ARITHMETIC
    Use TD_DrillStemComponents
    Use sROP_Variables
    

    
    IMPLICIT NONE
    
    INTEGER :: i , j , l
    INTEGER :: ifric
    INTEGER :: OldCasingPressure
    REAL :: PressBelowFloatValve , PressAboveFloatValve         ![psi]
    REAL :: PumpMinDischargedVol = 0.0050     ! [gal]
    REAL :: FloatValveBottomToUpAreaRatio = 1.1     ![-]
    REAL :: ZeroHeight , StaticHeadOnBit
    REAL(8) :: ShoeTVD
    !REAL(8) , DIMENSION(5) :: MDObserve , TVDObserve , StPressObserve , AnnPressObserve , NomMD
    
    ExitMass = 0.0
    BitPressLoss = 0.0
    
    WellHeadWasOpen = WellHeadOpen
    WellToChokeManifoldWasOpen = WellToChokeManifoldOpen
    KickWasExitingThroughChoke = .FALSE.
    
    
    IF (UtubeMode1Activated .OR. FloatValveWasOpen == .FALSE.) THEN       ! Horizontal line flow rate
        FlowEl(1 : NoHorizontalEl)%FlowRate = 0.0
    ELSE        ! connection and line is open
        FlowEl(1 : NoHorizontalEl)%FlowRate = StringFlowRate   ! pump flow rate [gpm]
    END IF
    
    !WRITE (*,*) 'a)A/B P Bit', StaticHeadOnBit , FlowEl(AnnulusFirstEl)%StartPress , SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff)
    !IF (FloatValveIn == .FALSE.)        FloatValveOpen = .TRUE.
    FloatValveWasOpen = FloatValveOpen
    
    PressBelowFloatValve = FlowEl(AnnulusFirstEl)%StartPress
    
    StMudVol = SUM(FlowEl(1 : StringLastEl)%Volume) * Convft3toUSGal
    StDeltaPtoDeltaVCompressibility = 1.0 / (MudCompressibility * StMudVol)
    
    AnnMudVol = SUM(FlowEl(AnnulusFirstEl : NumbEl)%Volume) * Convft3toUSGal
    !StCompressedMudVol = StCompressedMudVol + REAL(St_Saved_MudDischarged_Volume_Final)
    !WRITE (*,*) 'St_Saved_MudDischarged_Volume_Final', REAL(St_Saved_MudDischarged_Volume_Final)
    !StDeltaPDueToCompressibility = StCompressedMudVol / (MudCompressibility * StMudVol)
    !PressAboveFloatValve = StDeltaPDueToCompressibility + SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff) !!FlowEl(StringLastEl)%EndPress
    
    !IF (NoGasPocket > 0) THEN   ! mud exprience no comressibility
    !IF (KickVolume > 2.0) THEN
    IF ( (KickVolume > 2.0) .or. (NoGasPocket>1) .or. (any(FlowEl(OpenholeFirstEl:NumbEl)%Materialtype==1)) .or. (Rate_of_Penetration > 0.0) ) THEN
        AnnCompressedMudVol = 0.0
        AnnDeltaPDueToCompressibility = 0.0
        
    ELSE IF (WellHeadOpen) THEN
        AnnDeltaPtoDeltaVCompressibility = 1.0 / (MudCompressibility * AnnMudVol)
        
        AnnCompressedMudVol = BackPressure / AnnDeltaPtoDeltaVCompressibility
        AnnDeltaPDueToCompressibility = AnnCompressedMudVol / (MudCompressibility * AnnMudVol)
        
    ELSE        ! No gas pocket, wellhead is closed and mud is compressed based on volume pumped into annulus
        AnnDeltaPtoDeltaVCompressibility = 1.0 / (MudCompressibility * AnnMudVol)
        AnnCompressedMudVol = AnnCompressedMudVol + REAL(Ann_Saved_MudDischarged_Volume_Final)  !!!!!!!!!
        AnnCompressedMudVol = MAX((AnnCompressedMudVol - REAL(Qlost / ConvMinToSec / dt)) , 0.0)
        AnnDeltaPDueToCompressibility = AnnCompressedMudVol / (MudCompressibility * AnnMudVol)
    END IF    
    
    IF (FloatValveIn == .FALSE. .OR. NoGasPocket == 0 .OR. (FloatValveWasOpen .AND. REAL(St_Saved_MudDischarged_Volume_Final) >= PumpMinDischargedVol)) THEN     ! float valve remains open
        FloatValveOpen = .TRUE.
        
        FlowEl(StringFirstEl : StringLastEl)%FlowRate = REAL(St_Saved_MudDischarged_Volume_Final) / dt * ConvMinToSec       !StringFlowRate   ! String flow rate pump flow rate [gpm]
        
        !!!!!!!!!!!!!!! Calculating frictional pressure loss
        IF (WellHeadOpen) THEN
            DO ifric = 1 , StringLastEl
                CALL FricPressDrop(ifric)
                !WRITE (*,*) ' element No, FlowRate , Density, FricPressLoss', ifric, FlowEl(ifric)%FlowRate, FlowEl(ifric)%Density, FlowEl(ifric)%FricPressLoss    
                IF (IEEE_IS_NaN(FlowEl(ifric)%FricPressLoss)) THEN
                    WRITE (*,*) 'Hz/St start, end, density, Q, mu, Type' , FlowEl(ifric)%StartX, FlowEl(ifric)%EndX, FlowEl(ifric)%Density, FlowEl(ifric)%FlowRate, FlowEl(ifric)%mueff, FlowEl(ifric)%MaterialType
                    CALL ErrorStop('NaN in calculating pressure drop' , ifric)
                END IF
            END DO
        END IF
        
        !!!!!!!!!!!!!!!        
   
        
        !IF (ABS(MudVolume_InjectedToBH - St_Saved_MudDischarged_Volume_Final)> PumpMinDischargedVol)   WRITE (*,*) 'Injected to BH & St Saved Mud', MudVolume_InjectedToBH , St_Saved_MudDischarged_Volume_Final
        IF (BitTotallyPluged) THEN
            MudVolume_InjectedToBH = 0.d0
            StCompressedMudVol = StCompressedMudVol + REAL(St_Saved_MudDischarged_Volume_Final)
            StDeltaPDueToCompressibility = StCompressedMudVol * StDeltaPtoDeltaVCompressibility
            
        ELSE IF (WellHeadOpen .OR. NoGasPocket > 0) THEN
            IF (REAL(St_Saved_MudDischarged_Volume_Final) >= PumpMinDischargedVol) THEN
                
                MudVolume_InjectedToBH = St_Saved_MudDischarged_Volume_Final
                !WRITE (*,*) 'MudVolume_InjectedToBH,BitTrue', MudVolume_InjectedToBH
            
                !IF (BitTrue .AND. UtubeMode1Activated == .FALSE.) THEN
                IF (BitTrue) THEN
                    BitPressLoss = KBit * (MudVolume_InjectedToBH * ConvMinToSec / dt)**2
                    !WRITE (*,*) 'BitPressLoss', BitPressLoss
                END IF
            
                StCompressedMudVol = BitPressLoss / StDeltaPtoDeltaVCompressibility
                
            ELSE
                MudVolume_InjectedToBH = MAX( 0.d0 , REAL((StDeltaPDueToCompressibility + SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff - PressBelowFloatValve - AnnDeltaPDueToCompressibility - FloatValveMinOpenPressure) &
                                                        / StDeltaPtoDeltaVCompressibility ) * 1.d0))
                MudVolume_InjectedToBH = MIN(MudVolume_InjectedToBH , StCompressedMudVol)
                StCompressedMudVol = StCompressedMudVol - MudVolume_InjectedToBH
                
            END IF
            
            StDeltaPDueToCompressibility = StCompressedMudVol / (MudCompressibility * StMudVol)
        
        ELSE    ! IF (NoGasPocket == 0 .AND. WellHeadOpen == .FALSE.) THEN
            StCompressedMudVol = StCompressedMudVol + REAL(St_Saved_MudDischarged_Volume_Final)
            StDeltaPDueToCompressibility = StCompressedMudVol * StDeltaPtoDeltaVCompressibility
            MudVolume_InjectedToBH = MAX( 0.d0 , REAL((StDeltaPDueToCompressibility - AnnDeltaPDueToCompressibility - FloatValveMinOpenPressure) &
                                                        / (StDeltaPtoDeltaVCompressibility + AnnDeltaPtoDeltaVCompressibility)) * 1.d0)
            MudVolume_InjectedToBH = MIN(MudVolume_InjectedToBH , StCompressedMudVol)
            StCompressedMudVol = StCompressedMudVol - REAL(MudVolume_InjectedToBH)
            StDeltaPDueToCompressibility = StCompressedMudVol / (MudCompressibility * StMudVol)
            FlowEl(AnnulusFirstEl : NumbEl)%StartPress = FlowEl(AnnulusFirstEl : NumbEl)%StartPress + StDeltaPDueToCompressibility
            FlowEl(AnnulusFirstEl : NumbEl)%EndPress = FlowEl(AnnulusFirstEl : NumbEl)%EndPress + StDeltaPDueToCompressibility
            
            !WRITE (*,*) 'WellHeadOpen', WellHeadOpen
            !WRITE (*,*) ' StCompressedMudVol, StDeltaPDueToCompressibility',StCompressedMudVol, StDeltaPDueToCompressibility
            !WRITE (*,*) ' AnnCompressedMudVol, AnnDeltaPDueToCompressibility',AnnCompressedMudVol, AnnDeltaPDueToCompressibility
        END IF
        

         
        FlowEl(StringLastEl)%EndPress = FlowEl(AnnulusFirstEl)%StartPress + BitPressLoss + FloatValveMinOpenPressure
        !WRITE (*,*) 'BitPressLoss=', BitPressLoss
        FlowEl(StringLastEl)%StartPress = FlowEl(StringLastEl)%EndPress + FlowEl(StringLastEl)%FricPressLoss - FlowEl(StringLastEl)%StaticPressDiff
        DO i = StringLastEl - 1 , StringFirstEl , -1
            FlowEl(i)%EndPress = FlowEl(i + 1)%StartPress
            FlowEl(i)%StartPress = FlowEl(i)%EndPress + FlowEl(i)%FricPressLoss - FlowEl(i)%StaticPressDiff
            !WRITE(*,*) "STRING: Start , End Pressure", FlowEl(i)%StartPress , FlowEl(i)%EndPress
            !WRITE(*,*) "STRING: Start , End X", FlowEl(i)%StartX , FlowEl(i)%EndX
        END DO
        
        !FlowEl(NoHorizontalEl)%EndPress = FlowEl(StringFirstEl)%StartPress - FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD
        !WRITE (*,*) '- FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD1=', - FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD
        !FlowEl(NoHorizontalEl)%StartPress = FlowEl(NoHorizontalEl)%EndPress + FlowEl(StringLastEl)%FricPressLoss
        !DO i = NoHorizontalEl - 1 , 1 , -1
        !    FlowEl(i)%EndPress = FlowEl(i + 1)%StartPress
        !    FlowEl(i)%StartPress = FlowEl(i)%EndPress + FlowEl(i)%FricPressLoss
        !    !WRITE(*,*) "HORIZONTAL: Start , End Pressure", FlowEl(i)%StartPress , FlowEl(i)%EndPress
        !    !WRITE(*,*) "HORIZONTAL: Start , End X", FlowEl(i)%StartX , FlowEl(i)%EndX
        !END DO
        
        !!!!!!!!!!!!!!!!!!!!!!!!!!!! Float valve was open and remains open
        
    ELSE IF (REAL(St_Saved_MudDischarged_Volume_Final) < PumpMinDischargedVol) THEN     ! NoGasPocket > 0
        FloatValveOpen = FloatValveWasOpen      ! remains in its former status
        
        IF (FloatValveOpen) THEN
            PressAboveFloatValve = MAX(FlowEl(AnnulusFirstEl)%StartPress , SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff) + 0.052 * FlowEl(StringFirstEl)%Density * FlowEl(StringFirstEl)%StartTVD)
        ELSE
            PressAboveFloatValve = SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff) + StDeltaPDueToCompressibility
            MudVolume_InjectedToBH = 0.d0
        END IF
        
        
        IF (PressBelowFloatValve >= PressAboveFloatValve .AND. KickFlux) THEN       
            FloatValveOpen = .FALSE.
            IF (FloatValveOpen /= FloatValveWasOpen) THEN           ! float valve was open and now closed
                WRITE (*,*) 'Float valve was open and now closed'
                WRITE (*,*) 'PressAboveFloatValve=', PressAboveFloatValve
                WRITE (*,*) 'PressBelowFloatValve=', PressBelowFloatValve
            END IF
            
        END IF
        
        IF (FloatValveOpen) THEN
            MudVolume_InjectedToBH = MAX( 0.d0 , 0.1 * REAL((PressAboveFloatValve - PressBelowFloatValve - AnnDeltaPDueToCompressibility - FloatValveMinOpenPressure) &
                                                        / StDeltaPtoDeltaVCompressibility ) * 1.d0)
            MudVolume_InjectedToBH = MIN(MudVolume_InjectedToBH , StCompressedMudVol)
            !WRITE (*,*) 'MudVolume_InjectedToBH (No Pump)', MudVolume_InjectedToBH
            
            StCompressedMudVol = StCompressedMudVol - MudVolume_InjectedToBH
            StDeltaPDueToCompressibility = StCompressedMudVol / (MudCompressibility * StMudVol)
            !WRITE (*,*) 'StDeltaPDueToCompressibility(No Pump)', StDeltaPDueToCompressibility
            
        END IF
        
        IF (NoGasPocket == 0 .AND. WellHeadOpen == .FALSE.) THEN  !***********
            FlowEl(AnnulusFirstEl : NumbEl)%StartPress = FlowEl(AnnulusFirstEl : NumbEl)%StartPress + AnnDeltaPDueToCompressibility
            FlowEl(AnnulusFirstEl : NumbEl)%EndPress = FlowEl(AnnulusFirstEl : NumbEl)%EndPress + AnnDeltaPDueToCompressibility
        END IF
        
        
        
        IF (FloatValveOpen) THEN
            FlowEl(StringLastEl)%EndPress = MAX(FlowEl(AnnulusFirstEl)%StartPress , SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff) + 0.052 * FlowEl(StringFirstEl)%Density * FlowEl(StringFirstEl)%StartTVD)
        ELSE
            FlowEl(StringLastEl)%EndPress = StDeltaPDueToCompressibility + SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff)
        END IF
            
        FlowEl(StringLastEl)%StartPress = FlowEl(StringLastEl)%EndPress - FlowEl(StringLastEl)%StaticPressDiff
        DO i = StringLastEl - 1 , StringFirstEl , -1
            FlowEl(i)%EndPress = FlowEl(i + 1)%StartPress
            FlowEl(i)%StartPress = FlowEl(i)%EndPress + FlowEl(i)%FricPressLoss - FlowEl(i)%StaticPressDiff
            !WRITE(*,*) "STRING: Start , End Pressure", FlowEl(i)%StartPress , FlowEl(i)%EndPress
            !WRITE(*,*) "STRING: Start , End X", FlowEl(i)%StartX , FlowEl(i)%EndX
        END DO
        
        !FlowEl(NoHorizontalEl)%EndPress = FlowEl(StringFirstEl)%StartPress - FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD
        !WRITE (*,*) '- FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD2=', - FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD
        !FlowEl(NoHorizontalEl)%StartPress = FlowEl(NoHorizontalEl)%EndPress + FlowEl(StringLastEl)%FricPressLoss
        !DO i = NoHorizontalEl - 1 , 1 , -1
        !    FlowEl(i)%EndPress = FlowEl(i + 1)%StartPress
        !    FlowEl(i)%StartPress = FlowEl(i)%EndPress + FlowEl(i)%FricPressLoss
        !    !WRITE(*,*) "HORIZONTAL: Start , End Pressure", FlowEl(i)%StartPress , FlowEl(i)%EndPress
        !    !WRITE(*,*) "HORIZONTAL: Start , End X", FlowEl(i)%StartX , FlowEl(i)%EndX
        !END DO
        
        
        !WRITE (*,*) ' StCompressedMudVol, StDeltaPDueToCompressibility',StCompressedMudVol, StDeltaPDueToCompressibility
        !WRITE (*,*) ' AnnCompressedMudVol, AnnDeltaPDueToCompressibility',AnnCompressedMudVol, AnnDeltaPDueToCompressibility
        !!!!!!!!!!!!!!!!!!!!!!!!!!!! Float valve was open (close) and maybe remains open (close) or maybe closed
        
    ELSE IF(FloatValveWasOpen == .FALSE. .AND. REAL(St_Saved_MudDischarged_Volume_Final) >= PumpMinDischargedVol) THEN
        FloatValveOpen = .FALSE.
        MudVolume_InjectedToBH = 0.d0
        
        StCompressedMudVol = StCompressedMudVol + REAL(St_Saved_MudDischarged_Volume_Final)
        StDeltaPDueToCompressibility = StCompressedMudVol * StDeltaPtoDeltaVCompressibility
        PressAboveFloatValve = SUM(FlowEl(StringFirstEl : StringLastEl)%StaticPressDiff) + StDeltaPDueToCompressibility

        
        
        
        IF (PressAboveFloatValve > FloatValveBottomToUpAreaRatio * PressBelowFloatValve) THEN       ! float valve was open and now closed
            FloatValveOpen = .TRUE.
            WRITE (*,*) 'Float valve was closed and now opened'
            WRITE (*,*) 'PressAboveFloatValve=', PressAboveFloatValve
            WRITE (*,*) 'PressBelowFloatValve=', PressBelowFloatValve
        END IF
        
            
        FlowEl(StringLastEl)%EndPress = PressAboveFloatValve
        FlowEl(StringLastEl)%StartPress = FlowEl(StringLastEl)%EndPress - FlowEl(StringLastEl)%StaticPressDiff
        DO i = StringLastEl - 1 , StringFirstEl , -1
            FlowEl(i)%EndPress = FlowEl(i + 1)%StartPress
            FlowEl(i)%StartPress = FlowEl(i)%EndPress - FlowEl(i)%StaticPressDiff
            !WRITE(*,*) "STRING: Start , End Pressure", FlowEl(i)%StartPress , FlowEl(i)%EndPress
            !WRITE(*,*) "STRING: Start , End X", FlowEl(i)%StartX , FlowEl(i)%EndX
        END DO
        
        
        
  
    END IF
    
    IF ((UtubePossibility == .TRUE. .AND. Get_KellyConnection() /= KELLY_CONNECTION_STRING) .OR. NewPipeFilling == 0) THEN
        FlowEl(NoHorizontalEl)%EndPress = 0.0
    ELSE IF (WellHeadOpen == .FALSE.) THEN
        FlowEl(NoHorizontalEl)%EndPress = FlowEl(StringFirstEl)%StartPress - 0.052 * FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD
        !WRITE (*,*) 'Density , StartX= , StartPress', FlowEl(NoHorizontalEl)%Density , FlowEl(StringFirstEl)%StartTVD
    ELSE IF (WellHeadOpen) THEN
        FlowEl(NoHorizontalEl)%EndPress = FlowEl(StringFirstEl)%StartPress - 2.0 * 0.052 * FlowEl(NoHorizontalEl)%Density * FlowEl(StringFirstEl)%StartTVD
    END IF
    
    FlowEl(NoHorizontalEl)%StartPress = FlowEl(NoHorizontalEl)%EndPress + FlowEl(NoHorizontalEl)%FricPressLoss
    DO i = NoHorizontalEl - 1 , 1 , -1
        FlowEl(i)%EndPress = FlowEl(i + 1)%StartPress
        FlowEl(i)%StartPress = FlowEl(i)%EndPress + FlowEl(i)%FricPressLoss
        !WRITE(*,*) "HORIZONTAL: Start , End Pressure", FlowEl(i)%StartPress , FlowEl(i)%EndPress
        !WRITE(*,*) "HORIZONTAL: Start , End X", FlowEl(i)%StartX , FlowEl(i)%EndX
    END DO
        
    
    !WRITE (*,*) 'MudVolume_InjectedToBH==', MudVolume_InjectedToBH
    !WRITE (*,*) 'Ann_Saved_MudDischarged_Volume_Final==', Ann_Saved_MudDischarged_Volume_Final
    
!!!!!!!!!!!!!!!!!!!!! Pressure distribution in string and horizontal pump to string line
    
        
    IF (ShearBop_Situation_forTD == 1) THEN
        FlowEl(1 : NoHorizontalEl)%EndPress = 0.0
        FlowEl(1 : NoHorizontalEl)%StartPress = 0.0
        FlowEl(1 : NoHorizontalEl)%FricPressLoss = 0.0
    END IF
        
!!!!!!!!!!!!!!!!!!!!!!!!!
    
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    
    
    !IF (NoGasPocket == 0 .AND. WellHeadOpen) THEN
    !    FlowEl(1 : NoHorizontalEl + NoStringEl)%EndPress = FlowEl(1 : NoHorizontalEl + NoStringEl)%EndPress + StDeltaPDueToCompressibility + AnnDeltaPDueToCompressibility
    !    FlowEl(1 : NoHorizontalEl + NoStringEl)%StartPress = FlowEl(1 : NoHorizontalEl + NoStringEl)%StartPress + StDeltaPDueToCompressibility + AnnDeltaPDueToCompressibility
    !    FlowEl(NoHorizontalEl + NoStringEl +1 : NumbEl)%EndPress = FlowEl(NoHorizontalEl + NoStringEl +1 : NumbEl)%EndPress + AnnDeltaPDueToCompressibility
    !    FlowEl(NoHorizontalEl + NoStringEl + 1 : NumbEl)%StartPress = FlowEl(NoHorizontalEl + NoStringEl + 1 : NumbEl)%StartPress + AnnDeltaPDueToCompressibility
    !ELSE 
    !IF (NoGasPocket == 0 .AND. WellHeadOpen == .FALSE.) THEN
    !    FlowEl(1 : StringLastEl)%EndPress = FlowEl(1 : StringLastEl)%EndPress + StDeltaPDueToCompressibility + 30.0       ! badan eslah shavad
    !    FlowEl(1 : StringLastEl)%StartPress = FlowEl(1 : NoHorizontalEl + NoStringEl)%StartPress + StDeltaPDueToCompressibility + 30.0
    !    FlowEl(AnnulusFirstEl : NumbEl)%EndPress = FlowEl(AnnulusFirstEl : NumbEl)%EndPress + AnnDeltaPDueToCompressibility
    !    FlowEl(AnnulusFirstEl : NumbEl)%StartPress = FlowEl(AnnulusFirstEl : NumbEl)%StartPress + AnnDeltaPDueToCompressibility
    !END IF
    

    IF (UtubePossibility== .true. .and. Get_KellyConnection() /= KELLY_CONNECTION_STRING .and. WellHeadOpen) THEN
        MudVolume_InjectedToBH = 0.d0
        MudVolume_InjectedFromAnn = 0.d0
        
    !ELSE
    !    
    !    IF (FloatValveOpen .AND. WellHeadOpen .AND. NoGasPocket == 0) THEN
    !        MudVolume_InjectedToBH = MAX( 0.d0 , REAL((StDeltaPDueToCompressibility - FloatValveMinOpenPressure) / StDeltaPtoDeltaVCompressibility) * 1.d0)
    !        !MudVolume_InjectedFromAnn = Ann_Saved_MudDischarged_Volume_Final    !REAL((AnnDeltaPDueToCompressibility - BackPressure) / AnnDeltaPtoDeltaVCompressibility) * 1.d0
    !        !WRITE (*,*) 'Pressure above/under bit', FlowEl(NoHorizontalEl + NoStringEl)%EndPress, FlowEl(1 + NoHorizontalEl + NoStringEl)%StartPress
    !        IF (MudVolume_InjectedToBH <= 0)        MudVolume_InjectedToBH = 0.d0
    !        !IF (MudVolume_InjectedFromAnn <= 0)     MudVolume_InjectedFromAnn = 0.d0
    !    ELSE IF (FloatValveOpen .AND. WellHeadOpen == .FALSE. .AND. NoGasPocket == 0) THEN
    !        MudVolume_InjectedToBH = MAX( 0.d0 , REAL((PressAboveFloatValve + StDeltaPDueToCompressibility - AnnDeltaPDueToCompressibility - PressBelowFloatValve - FloatValveMinOpenPressure) &
    !                                                    / (StDeltaPtoDeltaVCompressibility - AnnDeltaPtoDeltaVCompressibility)) * 1.d0)
    !        MudVolume_InjectedFromAnn = 0.d0
    !    ELSE IF (FloatValveOpen .AND. WellHeadOpen .AND. NoGasPocket > 0) THEN
    !        MudVolume_InjectedToBH = MAX( 0.d0 , REAL((PressAboveFloatValve + StDeltaPDueToCompressibility - PressBelowFloatValve - FloatValveMinOpenPressure) / StDeltaPtoDeltaVCompressibility) * 1.d0)
    !    !    MudVolume_InjectedFromAnn = REAL(Ann_Saved_MudDischarged_Volume_Final) * 1.d0
    !    ELSE IF (FloatValveOpen .AND. WellHeadOpen == .FALSE. .AND. NoGasPocket > 0) THEN
    !        MudVolume_InjectedToBH = MAX( 0.d0 , REAL((PressAboveFloatValve + StDeltaPDueToCompressibility - AnnDeltaPDueToCompressibility - PressBelowFloatValve - FloatValveMinOpenPressure) &
    !                                                    / (StDeltaPtoDeltaVCompressibility - AnnDeltaPtoDeltaVCompressibility)) * 1.d0)
    !        MudVolume_InjectedFromAnn = 0.d0
    !    END IF
    !StCompressedMudVol = MAX(StCompressedMudVol - REAL(MudVolume_InjectedToBH) , 0.0)
    !AnnCompressedMudVol = MAX(AnnCompressedMudVol - REAL(MudVolume_InjectedFromAnn) , 0.0)
    !StDeltaPDueToCompressibility = StCompressedMudVol / (MudCompressibility * StMudVol)
    !AnnDeltaPDueToCompressibility = AnnCompressedMudVol / (MudCompressibility * AnnMudVol)
    END IF
    
  
    !MudVolume_InjectedToBH = 0.0
    !MudVolume_InjectedToBH = St_Saved_MudDischarged_Volume_Final 
    !WRITE (*,*) 'CompMudVol,      DeltaP,          MudVolumeInjected'
    !WRITE (*,*) StCompressedMudVol, StDeltaPDueToCompressibility, REAL(MudVolume_InjectedToBH)
    !WRITE (*,*) AnnCompressedMudVol, AnnDeltaPDueToCompressibility, REAL(MudVolume_InjectedFromAnn)
    !WRITE (*,*) 'Press above/Below Float valve ', FlowEl(NoHorizontalEl + NoStringEl)%EndPress , FlowEl(NoHorizontalEl + NoStringEl + 1)%StartPress
    !write(*,*) 'MudVolume_InjectedFromAnn***=' , MudVolume_InjectedFromAnn, Ann_Saved_MudDischarged_Volume_Final
                                                               
110     FORMAT (I6 , 4X , F6.2 , 7X , F4.2 , 3X , F4.1 , 2X , F4.2)
            
                
    !DO i = NumbEl , NumbEl - NoOpenHoleEl + 1 , -1         ! op elements
    !    WRITE (*,*) 'el no, start, end' , i, FlowEl(i)%StartPress, FlowEl(i)%EndPress
    !END DO
            
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            !!!!    Kick Information Reports
    
 
!!!!!!!!!!!!!!!!! 1- Stand pipe pressure gauge      PressureGauges(1)
    
    !ElementTrueDepth = STpipeGauge_Height/Convfttom
    !DistancetoRefrence = -170.7       ! 165 ft after pump and 100 ft before string
    i = 1
    DO WHILE (NOT(FlowEl(i)%EndX >= -170 .AND. FlowEl(i)%StartX <= -170))
        i = i + 1
        IF (i > NoHorizontalEl) EXIT
    END DO
    CALL PumpPressureDelay%AddToFirst(REAL(FlowEl(i)%StartPress - 0.052 * (STpipeGauge_Height / Convfttom) * FlowEl(i)%Density + (FlowEl(i)%StartX + 170) * FlowEl(i)%dPdLFric))
    CALL PumpPressureDelay%Remove(PressureTimeStepDelay(1) + 1)
    !IF (ANY(PUMP(:)%PowerFailMalf == 1))    PumpPressureDelay%Array(1 : PressureTimeStepDelay(1) / 2) = 0.0   !seyyed goft vaghti pumpfailure mishavad feshar dasti 0 nashavad, be in dalil in khat comment shod.
     
    DO j = PressureTimeStepDelay(1) , 1 , -1
        IF (NOT(IEEE_IS_NaN(PumpPressureDelay%Array(j)))) THEN
            PressureGauges(1) = INT(PumpPressureDelay%Array(j))
            EXIT
        END IF
    END DO
    
    !PressureGauges(1) = INT(PumpPressureDelay%Array(PressureTimeStepDelay(1)))
    
    IF (i > NoHorizontalEl) THEN
        WRITE (*,*) ' Error in calculating standpipe pressure '
    END IF
    
    IF (PressureGauges(1) < 0) THEN
        !CALL Set_StandPipePressure(real(PressureGauges(1) , 8))    ! for display console
        PressureGauges(1) = 0.0
        !CALL Set_StandPipePressure(0.0d0)   !StandPipePressureGauge = 0
    END IF
    DrillPipePressure = real(PressureGauges(1), 8)
    !WRITE (*,*) 'Drillpipe Pressure', PressureGauges(1)
    
!!!!!!!!!!!!!!!!! 2- Casing pressure gauge      PressureGauge(2)
    !WRITE (*,*) 'here 1', (WelltoPitsOpen == .FALSE. .AND. WellToChokeManifoldOpen) , (Valve(26)%Status == .TRUE. .AND. Valve(47)%Status == .TRUE. .AND. Valve(49)%Status == .TRUE.), BackPressure
    !WRITE (*,*) Valve(26)%Status , Valve(47)%Status , Valve(49)%Status
    !WRITE (*,*) (Valve(26)%Status == .TRUE.) , (Valve(47)%Status == .TRUE.) , (Valve(49)%Status == .TRUE.)
    
    !!! in normal mode changes in choke position immidiately observes in casing pressure
    !!      but when pumps off due to failure, casing pressure will drop after a delay time
    IF (WellToChokeManifoldOpen .OR. WellToChokeLineGauge) THEN
        !WRITE (*,*) 'Here 1'
        CALL CasingPressureDelay%AddToFirst(FlowEl(NoHorizontalEl + NoStringEl + NoAnnulusEl + NoWellToChokeEl)%EndPress)
        CALL CasingPressureDelay%Remove(PressureTimeStepDelay(1) + 1)
        
        DO j = 1 , PressureTimeStepDelay(1)
            IF (NOT(IEEE_IS_NaN(CasingPressureDelay%Array(j)))) THEN
                PressureGauges(2) = INT(CasingPressureDelay%Array(j))
                EXIT
            END IF
        END DO
        
        !PressureGauges(2) = INT(CasingPressureDelay%Array(1))
        IF (ANY(PUMP(:)%PowerFailMalf == 1)) THEN
            DO j = PressureTimeStepDelay(1) , 1 , -1
                IF (NOT(IEEE_IS_NaN(CasingPressureDelay%Array(j)))) THEN
                    PressureGauges(2) = INT(CasingPressureDelay%Array(j))
                    EXIT
                END IF
            END DO
        END IF
        
        
        
        
        !IF (ANY(PUMP(:)%PowerFailMalf == 1))    PressureGauges(2) = INT(CasingPressureDelay%Array(PressureTimeStepDelay(1)))
   
    ELSE !IF (ChokeLineGaugeToTanks) THEN
        PressureGauges(2) = 0
        !WRITE (*,*) 'Here 2'
    END IF

    
    !WRITE (*,*) 'GaugePoint(2)%Pressure =' , GaugePoint(2)%Pressure
    !IF (PressureGauges(2) < 0) THEN
    !    PressureGauges(2) = 0.0
    !END IF
    CALL Set_CasingPressure(real(PressureGauges(2) , 8))    ! for display console
    CasingPressureDownhole = real(PressureGauges(2) , 8)
    !IF (PressureGauges(2) > 3000.0) THEN
    !    !CALL Error(' High Casing Pressure')
    !END IF
    !WRITE (*,*) 'Casing Pressure=' , PressureGauges(2)
    
    

!!!!!!!!!!!!!!!!! 3- Bottom Hole Pressure       PressureGauge(3)
    
    CALL BottomHolePressureDelay%AddToFirst(FlowEl(OpenholeFirstEl)%StartPress)
    CALL BottomHolePressureDelay%Remove(PressureTimeStepDelay(2) + 1)
    !PressureGauges(3) = INT(BottomHolePressureDelay%Array(PressureTimeStepDelay(2)))

    DO j = PressureTimeStepDelay(2) , 1 , -1
        IF (NOT(IEEE_IS_NaN(BottomHolePressureDelay%Array(j)))) THEN
            PressureGauges(3) = INT(BottomHolePressureDelay%Array(j))
            EXIT
        END IF
    END DO
    
    
    BottomHolePress = BottomHolePressureDelay%Array(PressureTimeStepDelay(2))
    
    BottomHolePressure = REAL(PressureGauges(3) , 8)
!!!!!!!!!!!!!!!!! 4- Under Bit Pressure     PressureGauges(4)
        
    PressureGauges(4) = FlowEl(AnnulusFirstEl)%StartPress
    
!!!!!!!!!!!!!!!!!
    
    
!!!!!!!!!!!!!!!!! 5- Casing Shoe Pressure       PressureGauges(5)
    
    !IF (ShoeDepth <= FlowEl(NoHorizontalEl + NoStringEl + 1)%StartX) THEN
        DO ShoeFlowElNo = AnnulusFirstEl , NumbEl
            IF (FlowEl(ShoeFlowElNo)%StartX >= ShoeDepth .AND. FlowEl(ShoeFlowElNo)%EndX < ShoeDepth) EXIT
        END DO
        CALL TVD_Calculator(ShoeDepth , ShoeTVD)
        
        IF (ShoeFlowElNo > NumbEl) THEN
            WRITE (*,*) 'ShoeDepth =', ShoeDepth
            DO i = AnnulusFirstEl , NumbEl
                WRITE (*,*) 'i, StartX, EndX', i, FlowEl(i)%StartX, FlowEl(i)%EndX
            END DO
            
            CALL ErrorSTOP ('Error in finding location of shoe')
        END IF
        
    !ELSE
       ! WRITE (*,*) ' Error in calculating shoe pressure '
    !END IF
    
    CALL ShoePressureDelay%AddToFirst(REAL(FlowEl(ShoeFlowElNo)%StartPress &
                                        - (FlowEl(ShoeFlowElNo)%StartX - ShoeDepth) * FlowEl(ShoeFlowElNo)%dPdLfric &
                                        - (FlowEl(ShoeFlowElNo)%StartTVD - ShoeTVD) * FlowEl(ShoeFlowElNo)%dPdLGrav))
    CALL ShoePressureDelay%Remove(PressureTimeStepDelay(3) + 1)
    
    !FlowrateNearShoe = FlowEl(ShoeFlowElNo)%FlowRate
    DO j = PressureTimeStepDelay(3) , 1 , -1
        IF (NOT(IEEE_IS_NaN(ShoePressureDelay%Array(j)))) THEN
            PressureGauges(5) = INT(ShoePressureDelay%Array(j))
            EXIT
        END IF
    END DO
    
    
    !PressureGauges(5) = INT(ShoePressureDelay%Array(PressureTimeStepDelay(3)))
    ShoePressure = real(PressureGauges(5), 8)
        
    
    
    !IF (PressureGauges(5) >= FormationLostPressure)     WRITE (*,*) 'Near Shoe Flowrate', FlowEl(ShoeFlowElNo)%FlowRate
    
    ShoeMudViscosity = FlowEl(ShoeFlowElNo)%MuEff
    ShoeMudDensity = FlowEl(ShoeFlowElNo)%Density
    
    !WRITE (*,*) 'Drillstring speed (ft/s)' , DrillStringSpeed
    !WRITE (*,*) 'shoe mud speed ', FlowEl(ShoeFlowElNo - 1)%vel
    !WRITE (*,*) 'Shoe pressure (psi)', ShoePressureDelay%Array(1)
    
    
    
    
!!!!!!!!!!!!!!!!!
    
!!!!!!!!! 6- Pressure Before Bop 
    
    PressureGauges(6) = FlowEl(NoHorizontalEl + NoStringEl + NoAnnulusEl)%EndPress
    
!!!!!!!!!!!!!!!!!
101 FORMAT(4X, I2, 8X, (F8.1), 12X, (F8.3), 7X, (F8.2))
    
    !WRITE (*,*) ' Pump Pressure Delay', PumpPressureDelay%Array(1)
    !WRITE (*,*) ' Bottom Hole Pressure Delay', BottomHolePressureDelay%Array(1)
    !WRITE (*,*) ' Shoe Pressure Delay', ShoePressureDelay%Array(1)
    
    !IF (ALLOCATED(GasPocketWeight%Array) .AND. ChokeKroneckerDelta == 1) THEN
    !WRITE (*,*)  'Pocket No , Gas Pocket (psia) , Volume (gal) , Flow Induced (gpm) '
                                                                     
    !DO i = 1 , NoGasPocket
    !    WRITE (*,101) i, GasPocketNewPress%Array(i), GasPocketNewVol%Array(i) * ConvFt3toUSGal, GasPocketFlowInduced%Array(i)
    !END DO
   ! WRITE (*,*) 'Kchoke =', Kchoke, FlowEl(OpenholeFirstEl - 1)%FlowRate
    
    !DO i = 1 , NoGasPocket
    !    WRITE (*,*) 'Gas Kick Vol (gal)=' , GasPocketNewVol%Array(i) * ConvFt3toUSGal , GasPocketDeltaVol%Array(i) * ConvFt3toUSGal , GasPocketNewPress%Array(i)
    !END DO
    
    !WRITE (*,*) 'BHP (psig)=', BottomHolePress
    
    IF (ChokeKroneckerDelta == 1) THEN
        !WRITE (*,*) 'Casing Pressure' , PressureGauges(2)
        !WRITE (*,*) 'Below Bit' , FlowEl(AnnulusFirstEl)%StartPress
        !WRITE (*,*) 'Above Bit' , FlowEl(StringLastEl)%EndPress
        !WRITE (*,*) 'Pump Pressure' , PressureGauges(1)
        
        
        
        !WRITE (*,*) ' Kick Iteration', KickIteration
        !WRITE (*,*) ' Kchoke, Q =', Kchoke, FlowEl(j)%Flowrate  !, REAL((DeltaVolumePipe * ConvMinToSec / dt) + StringFlowRate)
        !DO l = NoHorizontalEl + NoStringEl + 1 , NumbEl
        !    WRITE (*,*) 'El No, Fric Press Loss, density , Q', l, FlowEl(l)%FricPressLoss, FlowEl(l)%StaticPressDiff, FlowEl(l)%Density, FlowEl(l)%Flowrate
        !END DO
        !write(*,*) 'BackPressure=' , BackPressure
        
        !WRITE (*,*) ' Kick Jacobian ', REAL(KickJacobian)
        !WRITE (*,*) ' KickVandPFunction = ' , REAL(-KickVandPFunction)
        !WRITE (*,*) ' Kick Unknown Vector = ' , REAL(KickUnknownVector)
        !WRITE (*,*) 'SUM(StaticPressDiff) , SUM(FricPressLoss)', SUM(FlowEl(GasPocketElementNo(1) : i)%FricPressLoss) , SUM(FlowEl(GasPocketElementNo(1) : i)%StaticPressDiff)
        !WRITE (*,*) 'Drillpipe, casing pressure', PressureGauges(1), PressureGauges(2)
    END IF
    
    !IF (NoWelltoChokeEl > 0 .AND. FlowEl(OpenholeFirstEl - 1)%MaterialType == 1 .AND. WellHeadOpen) THEN     ! kick is last element in choke line and does not exit.
    !    KickWasExitingThroughChoke = .TRUE.
    !    GasPocketDensity%Array(NoGasPocket) = (GasPocketweight%Array(NoGasPocket) / GasPocketModifiedVol%Array(NoGasPocket)) / convft3toUSgal     ! [lbm/ft^3 to ppg]
    !    ExitMass = (1.0 - (GasPocketModifiedVol%Array(NoGasPocket) / GasPocketNewVol%Array(NoGasPocket))) * GasPocketWeight%Array(NoGasPocket)      ! exit mass due to expand
    !    WRITE (*,*) ' ExitMass due to expand = ', GasPocketModifiedVol%Array(NoGasPocket) * Convft3ToUSgal , ExitMass
    !END IF
    
    !WRITE (*,*) 'Horiz 1'
    
    KickInFluxConditions = (FormationTop < TD_WellTotalVerticalLength) .AND. (NOT(InactiveInflux)) .AND. (FormPressure > BottomHolePress + 5.0)
    IF (KickInFluxConditions) THEN
        KickFlux = .TRUE.

        CALL NewGasKick
        
        !WRITE (*,*) 'Kick Flux top' , KickFlux
        !WRITE (*,*) 'FormPressure, BottomHolePress, FormationTop, TD_WellTotalVerticalLength' , FormPressure, BottomHolePress, FormationTop, TD_WellTotalVerticalLength
        
    ELSE
        IF (ALLOCATED(GasPocketWeight%Array) .AND. KickFlux) THEN
            KickOffBottom = .TRUE.
            WRITE (*,*) 'Kick Off Bottom'
            WRITE (*,*) 'FormPressure , BottomHolePress' , FormPressure , BottomHolePress
            !WRITE (*,*) 'No  Press(psia)  Vol(gal)  Weight(lbm)  Flow Induced(gpm)  Flow El Press(psia)'                                                                                                
            DO i = 1 , NoGasPocket
                WRITE (*,102) i , GasPocketNewPress%Array(i), GasPocketNewVol%Array(i) * Convft3toUSgal, GasPocketWeight%Array(i), GasPocketFlowInduced%Array(i), FlowEl(GasPocketFlowEl(i , 1))%StartPress + StandardPress
            END DO
        END IF
        KickFlux = .FALSE.
    END IF
    
    IF (ALLOCATED(KickJacobian))        OldKickJacobian = KickJacobian
    
102 FORMAT (I2, 3X, (F8.1), 2X, (F8.2), 2X, (F8.3), 8X, (F8.2), 10X, (F8.1))
    
    !!!!!!!!  Auto Choke Procedure
    ! DO i = 1 , 5
    !   AreaChange = -1.0 * (BottomHolePressure - (FormPressure + BHPSafetyMargin)) / FlowEl(OpenholeFirstEl - 1)%Flowrare**2 * 89158.0 &
    !                       * (0.26 * 0.61)**2 * TotalOpenChokeArea**3 / (4.0 * ChokeDensity)
    !   CHOOKE(1)%AreaChokeFinal = CHOOKE(1)%AreaChokeFinal + AreaChange / * Convfttoinch**2
    !
    !
    !
    ! END DO
    !
    !
    !
    !
    !
    
    
    !WRITE (*,*) ' SecondaryKickWeight', SecondaryKickWeight
    !WRITE (*,*) ' SecondaryKickVol', SecondaryKickVol
    SecondKickVolume = SecondaryKickVol
    
    
    
    IF (WellHeadOpen == .FALSE. .OR. (FlowEl(OpenholeFirstEl - 1)%Flowrate < PressFlowrateTolerance .AND. FlowEl(AnnulusLastEl)%Flowrate < PressFlowrateTolerance)) THEN
        OnShakerDensity = 0.0
    ELSE IF (FlowEl(OpenholeFirstEl - 1)%MaterialType == 1 .AND. ChokeKroneckerDelta == 1) THEN
        OnShakerDensity = 2.0
    ELSE IF (ChokeKroneckerDelta == 0) THEN
        OnShakerDensity = FlowEl(AnnulusLastEl)%Density
    ELSE IF (ChokeKroneckerDelta == 1) THEN  !!!(FlowEl(OpenholeFirstEl - 1)%Flowrate > PressFlowrateTolerance .AND. FlowEl(AnnulusLastEl)%Flowrate < PressFlowrateTolerance) THEN
        OnShakerDensity = FlowEl(OpenholeFirstEl - 1)%Density
    ELSE
        OnShakerDensity = (FlowEl(OpenholeFirstEl - 1)%Density * FlowEl(OpenholeFirstEl - 1)%Flowrate &
                        + FlowEl(AnnulusLastEl)%Density * FlowEl(AnnulusLastEl)%Flowrate) / (FlowEl(OpenholeFirstEl - 1)%Flowrate + FlowEl(AnnulusLastEl)%Flowrate)
    END IF
    !WRITE (*,*) 'ANINT(OnShakerDensity * 100) / 100', ANINT(OnShakerDensity * 100) / 100 , OnShakerDensity
    CALL Set_MudWeightOut(ANINT(OnShakerDensity * 100) / 100)

    
    IF (ALLOCATED(FinalFlowEl))     DEALLOCATE(FinalFlowEl)
    ALLOCATE(FinalFlowEl(NumbEl))
    
    FinalFlowEl(:)%StartX = FlowEl(:)%StartX
    FinalFlowEl(:)%EndX = FlowEl(:)%EndX
    FinalFlowEl(:)%StartTVD = FlowEl(:)%StartTVD
    FinalFlowEl(:)%EndTVD = FlowEl(:)%EndTVD
    FinalFlowEl(:)%Length = FlowEl(:)%Length
    FinalFlowEl(:)%DepthDiff = FlowEl(:)%DepthDiff
    FinalFlowEl(:)%density = FlowEl(:)%density
    FinalFlowEl(:)%StartPress = FlowEl(:)%StartPress
    FinalFlowEl(:)%EndPress = FlowEl(:)%EndPress
    FinalFlowEl(:)%dPdLFric = FlowEl(:)%dPdLFric
    FinalFlowEl(:)%dPdLGrav = FlowEl(:)%dPdLGrav

    !WRITE (*,*) 'FlowRate=', FlowEl(AnnulusFirstEl)%FlowRate
    !WRITE (*,*) 'Pressure Loss in Drill String', SUM(FlowEl(StringFirstEl : StringLastEl)%FricPressLoss)
    !WRITE (*,*) 'Pressure Loss in Annulus', SUM(FlowEl(AnnulusFirstEl : AnnulusLastEl)%FricPressLoss)
    !MDObserve(:) = [3000.0 , 4349.0 , 11880.0 , 19880.0 , 21680.0]
    !NomMd (:) = [3000 , 4298 , 11690 , 19690 , 21490]
    !DO i = 1 , 5
    !    CALL TVD_Calculator(MDObserve(i) , TVDObserve(i))
    !END DO
    !
    !DO i = 1 , 5
    !    
    !    WRITE (*,*) 'MDObserve(i)', INT(NomMD(i))
    !    WRITE (*,*) 'TVDObserve(i)', INT(TVDObserve(i))
    !        
    !    DO j = StringFirstEl , StringLastEl
    !        IF (INT(MDObserve(i)) < INT(FinalFlowEl(j)%EndX)) EXIT
    !    END DO
    !    StPressObserve(i) = FlowEl(j)%StartPress - (MDObserve(i) - FlowEl(j)%StartX) * FlowEl(j)%dPdLfric + (TVDObserve(i) - FlowEl(j)%StartTVD) * FlowEl(j)%dPdLGrav
    !    WRITE (*,*) 'String Pressure', INT(StPressObserve(i))
    !    
    !    
    !    
    !    IF (INT(MDObserve(i)) <= INT(FlowEl(AnnulusFirstEl)%StartX)) THEN         !! mouse pointer is in the annulus space
    !        DO j = AnnulusFirstEl , AnnulusLastEl
    !            IF (INT(FlowEl(j)%EndX) <= INT(MDObserve(i)))   EXIT
    !        END DO
    !    ELSE IF (INT(MDObserve(i)) > INT(FinalFlowEl(NumbEl)%EndX)) THEN                 ! mouse pointer is in the open hole space
    !        DO j = OpenholeFirstEl , NumbEl
    !            IF (INT(FinalFlowEl(j)%EndX) <= INT(MDObserve(i)))   EXIT
    !        END DO
    !    END IF
    !    AnnPressObserve(i) = FlowEl(j)%StartPress - (FlowEl(j)%StartX - MDObserve(i)) * FlowEl(j)%dPdLfric &
    !                            - (FlowEl(j)%StartTVD - TVDObserve(i)) * FlowEl(j)%dPdLGrav
    !    WRITE (*,*) 'Annulus Pressure', INT(AnnPressObserve(i))
    !    
    !    
    !END DO
    

    
    
    END SUBROUTINE
        
    
    
SUBROUTINE SOLVE_LINEAR_EQUATIONS(A , x , b , error, dim)
    
    !!! This subroutine solves a linear systems of equations Ax=b
    !! if vaiable erorr changed its value to .FALSE. means that the system of equations cab not be solved
    !! I use this subroutine to solve the linearized equations which uprising in calculation of volume and pressure of gas kick pockets
    
    USE KickVARIABLES

    IMPLICIT NONE
    INTEGER , INTENT(IN) :: dim
    REAL(8) , DIMENSION(dim,dim) , INTENT(in) :: A
    REAL , DIMENSION(dim) , INTENT(OUT) :: x
    REAL(8) , DIMENSION(dim) , INTENT(in) :: b
    LOGICAL , INTENT(OUT) :: error
    REAL(8) , DIMENSION(:,:) , ALLOCATABLE :: m
    INTEGER , DIMENSION(1) :: max_loc
    REAL(8) , DIMENSION(:) , ALLOCATABLE :: temp_row
    INTEGER :: n , k
    
    !WRITE (*,*) 'SIZE(A , dim = 1), SIZE(A , dim = 2), SIZE(b)', SIZE(A , dim = 1), SIZE(A , dim = 2), SIZE(b)
    error = (SIZE(A , dim = 1) /= SIZE(b)) .OR. (SIZE(A , dim = 2) /= SIZE(b))
    !WRITE (*,*) 'SOLVE_LINEAR_EQUATIONS 1' , error
    
    IF (error) THEN
        x = 0.0d0
        RETURN
    END IF
    n = SIZE(b)
    
    ALLOCATE (m(n , n + 1) , temp_row(n + 1))
    m(1:n , 1:n) = A
    m(1:n , n + 1) = b
    !WRITE (*,*) 'SOLVE_LINEAR_EQUATIONS 2' , m
    ! Triangularization phase
    TRIANG_LOOP: DO k = 1 , n
        max_loc = MAXLOC(ABS(m(k:n , k)))
        temp_row(k:n + 1) = m(k , k:n + 1)
        m(k , k:n+1) = m(k-1+max_loc(1) , k:n+1)
        m(k - 1 + max_loc(1) , k:n + 1) = temp_row(k:n + 1)
        !WRITE (*,*) 'SOLVE_LINEAR_EQUATIONS 3' , max_loc
        
        IF (m(k , k) == 0) THEN
            error = .TRUE.
            !WRITE (*,*) 'SOLVE_LINEAR_EQUATIONS 4'
            EXIT TRIANG_LOOP
        ELSE
            !WRITE (*,*) 'SOLVE_LINEAR_EQUATIONS 5'
            m(k , k : n + 1) = m(k , k : n + 1) / m(k , k)
            m(k + 1 : n , k + 1 : n + 1) = m(k + 1 : n , k + 1 : n + 1) - SPREAD(m(k , k + 1:n + 1) , 1, n - k) * SPREAD(m(k + 1:n , k) , 2 , n - k + 1)
        END IF
    END DO TRIANG_LOOP
    !WRITE (*,*) 'SOLVE_LINEAR_EQUATIONS 6'
    
    ! Back substitution phase
    IF (error) THEN
        x = 0.0
    ELSE
        DO k = n , 1 , -1
            x(k) = REAL(m(k , n + 1) - SUM(m(k , k + 1 : n) * x(k + 1 : n)))
            !WRITE (*,*) 'SOLVE_LINEAR_EQUATIONS 7'
        END DO
    END IF
    
    DEALLOCATE(m , temp_row)
    
END SUBROUTINE solve_linear_equations
    